<?php

  /**
   * Importer module definition
   *
   * @package activeCollab.modules.importer
   * @subpackage models
   */
  class ImporterModule extends Module {
    
    /**
     * Plain module name
     *
     * @var string
     */
    var $name = 'importer';
    
    /**
     * Is system module flag
     *
     * @var boolean
     */
    var $is_system = false;
    
    /**
     * Module version
     *
     * @var string
     */
    var $version = '1.0';
    
    // ---------------------------------------------------
    //  Events and Routes
    // ---------------------------------------------------
    
    /**
     * Define module routes
     *
     * @param Router $r
     * @return null
     */
    function defineRoutes(&$router) {
    	$router->map('admin_importer', 'admin/importer', array('controller' => 'importer_admin'));
    	$router->map('admin_importer_create_backup', 'admin/importer/create-backup', array('controller' => 'importer_admin', 'action' => 'create_backup'));
    	$router->map('admin_importer_import_companies', 'admin/importer/import-companies', array('controller' => 'importer_admin', 'action' => 'import_companies'));
    	$router->map('admin_importer_import_categories', 'admin/importer/import-categories', array('controller' => 'importer_admin', 'action' => 'import_categories'));
    	$router->map('admin_importer_import_project', 'admin/importer/import-project', array('controller' => 'importer_admin', 'action' => 'import_project'));
    } // defineRoutes
    
    /**
     * Define event handlers
     *
     * @param EventsManager $events
     * @return null
     */
    function defineHandlers(&$events) {
      $events->listen('on_admin_sections', 'on_admin_sections');
    } // defineHandlers
    
    /**
     * Returns true if this module can be installed
     *
     * @param array $log
     * @return boolean
     */
    function canBeInstalled(&$log) {
      if(strnatcmp(phpversion(), '5.0.0') <= 0) {
        $log[] = lang('Your PHP version is older than PHP 5.0.0 and you won\'t be able to use Importer module. Please upgrade to at least PHP version');
        return false;
      } // if
      
      return true;
    } // canBeInstalled
    
    /**
     * Get module display name
     *
     * @return string
     */
    function getDisplayName() {
      return lang('Importer');
    } // getDisplayName
    
    /**
     * Return module description
     *
     * @param void
     * @return string
     */
    function getDescription() {
      return lang('Import data from Basecamp');
    } // getDescription
    
  }

?>