<?php

	// we need admin controller
  use_controller('admin', SYSTEM_MODULE);
  
  /**
   * Importer admin controller
   *
   * @package activeCollab.modules.importer
   * @subpackage controllers
   */
  class ImporterAdminController extends AdminController  {
  	
  	/**
     * Controller name
     *
     * @var string
     */
    public $controller_name = 'importer_admin';
  	
  	/**
     * Active module
     *
     * @var string
     */
    protected $active_module = IMPORTER_MODULE;
    
    /**
     * Importer instance
     *
     * @var Importer
     */
    protected $importer;
    
    /**
     * Mapped imported companies
     *
     * @var array
     */
    protected $imported_companies_map = array();
    
    /**
     * Mapped imported users
     *
     * @var array
     */
    protected $imported_users_map = array();
    
    /**
		 * File handle
		 *
		 * @var boolean
		 */
		protected $file_handle;
		
		/**
		 * Filename which containing project data
		 *
		 * @var string
		 */
		protected $import_project_filename;
		
		/**
		 * Cached imported project ID
		 *
		 * @var integer
		 */
		protected $imported_project_id;
		
		/**
		 * Mapping Project Objects
		 *
		 * @var array
		 */
		protected $project_objects_map = array();
		
		/**
		 * Find Ticket / Checklist number
		 *
		 * @var integer
		 */
		protected $counter = 1;
		
		/**
		 * If Fileinfo extension loaded
		 *
		 * @var boolean
		 */
		protected $fileinfo_loaded = true;
		
		/**
		 * Basecamp exported files info
		 *
		 * @var array
		 */
		protected $info = array();
		
		/**
		 * Show import form
		 *
		 * @var boolean
		 */
		protected $show = true;
    
    /**
     * Constructor method
     *
     * @param string $request
     * @return ImporterAdminController
     */
    function __construct($request) {
      parent::__construct($request);
      
      if(strnatcmp(phpversion(), '5.0.0') <= 0) {
        $this->wireframe->addPageMessage(lang("<b>Your PHP version is older than PHP 5.0.0</b> - You won't be able to use Importer module. Please upgrade to at least PHP version to solve this issue."), PAGE_MESSAGE_WARNING);
      } // if
      
      if(!($this->importer instanceof Importer)) {
      	$this->importer = new Importer();
      } // if
      
      if(!$this->importer->checkFiles()) {
      	$this->wireframe->addPageMessage(lang("There are no files ready for import in the following folder <strong>:folder</strong>", array('folder' => IMPORT_PATH)), PAGE_MESSAGE_WARNING);
    		$this->show = false;
      } // if
      
      $this->info = $this->importer->getImportFile('info');
      
      if($this->info['ac_license_type'] == 'corporate' && LICENSE_PACKAGE == 'smallbiz') {
      	$this->wireframe->addPageMessage(lang("You can't import the data exported for activeCollab :exported_for into your :importing_in package", array('exported_for' => ucfirst($this->info['ac_license_type']), 'importing_in' => ucfirst(LICENSE_PACKAGE))), PAGE_MESSAGE_WARNING);
    		$this->show = false;
      } // if
      
      // check whether required modules are installed
      $missing_modules = array();

      if(!module_loaded(DISCUSSIONS_MODULE)) {
    		$missing_modules[] = 'Discussions';
    	} // if
    	
      if(!module_loaded(FILES_MODULE)) {
    		$missing_modules[] = 'Files';
    	} // if
    	
      if($this->info['ac_license_type'] == 'corporate') {
      	if(!module_loaded(TICKETS_MODULE)) {
      		$missing_modules[] = 'Tickets';
      	} // if
      	
      	if(!module_loaded(TIMETRACKING_MODULE)) {
      		$missing_modules[] = 'Time Tracking';
      	} // if
      } else {
      	if(!module_loaded(CHECKLISTS_MODULE)) {
      		$missing_modules[] = 'Checklists';
      	} // if
      } // if
      
      if(is_foreachable($missing_modules)) {
      	$this->wireframe->addPageMessage(lang("Please install the following missing modules first in order to be able to import the data: <strong>:missing_modules</strong>.", array('missing_modules' => implode(', ', $missing_modules))), PAGE_MESSAGE_WARNING);
    		$this->show = false;
      } // if

      $this->wireframe->addBreadCrumb(lang('Importer'), assemble_url('admin_importer'));
      
      if(!extension_loaded('fileinfo')) {
      	$this->fileinfo_loaded = false;
      } // if
      
      $this->smarty->assign('show', $this->show);
    } // __construct
    
    /**
     * Import data index page
     *
     * @param void
     * @return null
     */
    function index() {
  		$default_categories = null;
  		if(is_foreachable($import_categories = $this->importer->getImportFile('default_categories'))) {
  			if(!empty($_SESSION['imported_categories']) && $_SESSION['imported_categories'] === true) {
					$import_categories['imported'] = true;
				} else {
					$import_categories['imported'] = false;
				} // if
				
				$default_categories = $import_categories;
  		} // if
  		
  		$companies = null;
  		if(is_foreachable($import_companies = $this->importer->getImportFile('companies'))) {
  			if(!empty($_SESSION['imported_companies']) && $_SESSION['imported_companies'] === true) {
					$import_companies['imported'] = true;
				} else {
					$import_companies['imported'] = false;
				} // if
				
				$companies = $import_companies;
  		} // if
  		
  		$projects = array(); $imoprted_projects_count = 0;
  		if(is_foreachable($import_projects = $this->importer->getImportFile('projects_list'))) {
  			foreach($import_projects as $import_project) {
  				$project = $import_project;
  				if(!empty($_SESSION['imported_projects']) && in_array($import_project['id'], $_SESSION['imported_projects'])) {
  					$project['imported'] = true;
  					$imported_projects_count++;
  				} else {
  					$project['imported'] = false;
  				} // if
  				
  				$projects[] = $project;
  			} // foreach
  		} // if
  		
    	$this->smarty->assign(array(
    		'info'               	=> $this->info,
    		'default_categories' 	=> $default_categories,
    		'companies'          	=> $companies,
    		'projects'           	=> $projects,
    		'role_data'          	=> array('admin_role_id' => 1, 'can_see_private_role_id' => 3, 'cannot_see_private_role_id' => ConfigOptions::getValue('default_role')),
    		'all_imported'				=> count($projects) == $imported_projects_count ? true : false
    	));
    	
    	js_assign(array(
    		'create_backup_url' => assemble_url('admin_importer_create_backup'),
    		'import_companies_url'  => assemble_url('admin_importer_import_companies'),
    		'import_categories_url' => assemble_url('admin_importer_import_categories'),
    		'import_project_url'    => assemble_url('admin_importer_import_project'),
    		'is_companies_imported' => $import_companies['imported']
    	));
    } // index
    
    /**
     * Create database backup first
     *
     * @param void
     * @return null
     */
    function create_backup() {
    	if(!$this->request->isAsyncCall() || !$this->request->isSubmitted()) {
    		die('HTTP/1.1 400 Bad Request');
    	} // if
    	
      if(is_dir(WORK_PATH)) {
        if(is_writable(WORK_PATH)) {
          $connection =& DBConnection::instance();
          $tables = $connection->listTables(TABLE_PREFIX);
          if(is_foreachable($tables)) {
            $dump = $connection->dumpTables($tables, WORK_PATH . '/database-backup-' . date('Y-m-d-H-i-s') . '.sql', true, true);
            
            if($dump && !is_error($dump)) {
              die(lang('ok'));
            } else {
              if(is_error($dump)) {
              	die($dump->getMessage());
              } else {
              	die(lang('Failed to back up database content'));
              } // if
            } // if
          } else {
            die(lang('There are no activeCollab tables in the database'));
          } // if
        } else {
          die(lang('Work folder not writable'));
        } // if
      } else {
        die(lang('Work folder not found. Expected location: :work_path', array('work_path' => WORK_PATH)));
      } // if
    } // create_backup
    
    /**
     * Import companies
     *
     * @param void
     * @return null
     */
    function import_companies() {
    	if(!$this->request->isAsyncCall() || !$this->request->isSubmitted()) {
    		die('HTTP/1.1 400 Bad Request');
    	} // if
    	
    	$role_data = $this->request->post('role');
    	
    	if(is_foreachable($import_companies = Importer::getImportFile('companies'))) {
    		$ac_companies = $this->importer->getAcCompanies();
    		foreach($import_companies as $import_company) {
    			
    			if(!in_array(array_var($import_company, 'name'), $ac_companies)) {
	    			if(array_var($import_company, 'is_owner')) {
	    				$owner_company_id = $this->importer->getOwnerCompanyId();
	    				$this->imported_companies_map[array_var($import_company, 'id')] = $owner_company_id;
	    				
	    				// import owner company users
	    				$this->import_users($import_company, $owner_company_id, $role_data);
	    			} else {
	    				// import client company
	    				try {
								$this->importer->prepare_and_execute('companies', array(
									'name'					=> array_var($import_company, 'name'),
									'created_on'		=> array_var($import_company, 'created_on', new DateTimeValue()),
									'is_owner'			=> array_var($import_company, 'is_owner', false)
								));
							} catch(Exception $e) {
								die(lang('Failed to import client company :name', array('name' => array_var($import_company, 'name'))));
							} // try
	    				
	    				$imported_company_id = db_last_insert_id();
	    				
	    				// import additional client company data such as office address, phone, fax, etc.
	    				if(is_foreachable(array_var($import_company, 'company_details'))) {
	    					foreach(array_var($import_company, 'company_details') as $name => $value) {
	    						if($value != null) {
				    				try {
											$this->importer->prepare_and_execute('company_config_options', array(
												'company_id'		=> $imported_company_id,
												'name'					=> $name,
												'value'					=> serialize($value)
											));
										} catch(Exception $e) {
											die(lang('Failed to import additional client company data :name', array('name' => array_var($import_company, 'name'))));
										} // try
	    						} // if
	    					} // foreach
	    				} // if
	    				
	    				$this->imported_companies_map[array_var($import_company, 'id')] = $imported_company_id;
	    				
	    				// import client company users
	    				$this->import_users($import_company, $imported_company_id, $role_data);
	    			} // if
    			} else {
    				$existing_company_id = $this->importer->getCompanyIdByName(array_var($import_company, 'name'));
    				$this->imported_companies_map[array_var($import_company, 'id')] = $existing_company_id;
    				
    				// import users in already created company in activeCollab
    				$this->import_users($import_company, $existing_company_id, $role_data);
    			} // if
    		} // foreach
    		
    		// create companies map file
    		if(is_foreachable($this->imported_companies_map)) {
					$result = file_put_contents(IMPORT_PATH . '/companies_map.php', "<?php return " . var_export($this->imported_companies_map, true) . " ?>");
					if($result) {
						@chmod(IMPORT_PATH . '/companies_map.php', 0777);
					} else {
						die(lang('Failed to create companies map file: /import/companies_map.php'));
					} // if
				} // if
				
				// create users map file
				if(is_foreachable($this->imported_users_map)) {
					$result = file_put_contents(IMPORT_PATH . '/users_map.php', "<?php return " . var_export($this->imported_users_map, true) . " ?>");
					if($result) {
						@chmod(IMPORT_PATH . '/users_map.php', 0777);
					} else {
						die(lang('Failed to create users map file: /import/users_map.php'));
					} // if
				} // if
    		
    		$_SESSION['imported_companies'] = true;
    		
    		die(lang('ok'));
    	} else {
    		die(lang('There are no companies to import'));
    	} // if
    } // import_companies
    
    /**
     * Import users
     *
     * @param array $import_company
     * @param integer $ac_company_id
     * @param array $role_data
     * @return null
     */
    function import_users($import_company, $ac_company_id, $role_data) {
    	$users_table = TABLE_PREFIX . 'users';
	    $user_config_options_table = TABLE_PREFIX . 'user_config_options';
    	
    	if(is_foreachable(array_var($import_company, 'users'))) {	
    		$ac_users = $this->importer->getAcUsers();
	    	foreach(array_var($import_company, 'users') as $import_company_user) {
    			if(!in_array(strtolower(array_var($import_company_user, 'email')), $ac_users)) {
    				// set appropriate system role
    				$role_id = null;
    				if(array_var($import_company_user, 'is_administrator') === true) {
    					$role_id = array_var($role_data, 'admin_role_id');
    				} elseif(array_var($import_company_user, 'can_see_private_objects') === true) {
    					$role_id = array_var($role_data, 'can_see_private_role_id');
    				} elseif(array_var($import_company_user, 'can_see_private_objects') === false) {
    					$role_id = array_var($role_data, 'cannot_see_private_role_id');
    				} // if
    				
    				$password = sha1(LICENSE_KEY . array_var($import_company_user, 'username'));
    				
    				// check if user has auto access to new projects
    				$auto_assign = 0; $auto_assign_role_id = null; $auto_assign_permissions = null;
    				if(array_var($import_company_user, 'auto_assign') != false) {
    					$auto_assign = array_var($import_company_user, 'auto_assign');
	    				$auto_assign_role_id = array_var($import_company_user, 'auto_assign_role_id');
	    				$auto_assign_permissions = serialize(array_var($import_company_user, 'auto_assign_permissions'));
    				} // if
    				
    				$email = array_var($import_company_user, 'email', null);
    				$first_name = array_var($import_company_user, 'first_name', null);
    				$last_name = array_var($import_company_user, 'last_name', null);
    				
    				// import user
    				try {
							$this->importer->prepare_and_execute('users', array(
								'company_id'								=> $ac_company_id,
								'role_id'										=> $role_id,
								'first_name'								=> $first_name,
								'last_name'									=> $last_name,
								'email'											=> $email,
								'password'									=> $password,
								'token'											=> make_string(40),
								'created_on'								=> array_var($import_company_user, 'created_on', new DateTimeValue()),
								'auto_assign'								=> $auto_assign,
								'auto_assign_role_id'				=> $auto_assign_role_id,
								'auto_assign_permissions'		=> $auto_assign_permissions
							));
						} catch(Exception $e) {
							die(lang('Failed to import user :name', array('name' => $first_name . ' ' . $last_name)));
						} // try
    				
    				$imported_user_id = db_last_insert_id();
    				
    				// update user search index
    				try {
    					$this->importer->updateSearchIndex($imported_user_id, 'User', array(
    						'email' 			=> $email,
    						'first_name' 	=> $first_name,
    						'last_name' 	=> $last_name
    					));
    				} catch(Exception $e) {
    					die(lang('Failed to update search index for user :name', array('name' => $first_name . ' ' . $last_name)));
    				} // try
    				
    				// import additional user data such as timezone, title, etc.
    				if(is_foreachable(array_var($import_company_user, 'user_details'))) {
    					foreach(array_var($import_company_user, 'user_details') as $name => $value) {
    						if($value != null) {
			    				try {
										$this->importer->prepare_and_execute('user_config_options', array(
											'user_id'		=> $imported_user_id,
											'name'			=> $name,
											'value'			=> serialize($value)
										));
									} catch(Exception $e) {
										die(lang('Failed to import additional user data :name', array('name' => array_var($import_company_user, 'first_name') . array_var($import_company_user, 'last_name'))));
									} // try
    						} // if
    					} // foreach
    				} // if
    				
    				$this->imported_users_map[array_var($import_company_user, 'id')] = $imported_user_id;
    			} else {
    				$existing_user_id = $this->importer->getUserIdByEmail(array_var($import_company_user, 'email'));
    				$this->imported_users_map[array_var($import_company_user, 'id')] = $existing_user_id;
    			} // if
    		} // foreach
    	} // if
    } // import_users
    
    /**
     * Import master categories
     *
     * @param void
     * @return null
     */
    function import_categories() {
    	if(!$this->request->isAsyncCall() || !$this->request->isSubmitted()) {
    		die('HTTP/1.1 400 Bad Request');
    	} // if
    	
    	if(is_foreachable($import_categories = Importer::getImportFile('default_categories'))) {
    		foreach($import_categories as $module => $category_names) {
    			$ac_categories = $this->importer->getAcCategories($module);
    			$categories = array_unique(array_merge($ac_categories, $category_names));
    			
    			// import master categories
    			try {
						$this->importer->updateMasterCategories($categories, $module);
					} catch(Exception $e) {
						die(lang('Failed to import master categories'));
					} // try
    		} // foreach
    		
    		$_SESSION['imported_categories'] = true;
    		
    		die(lang('ok'));
    	} else {
    		die(lang('There are no master categories to import'));
    	} // if
    } // import_categories
    
    /**
     * Import project and all related objects (milestones, tickets, discussion, etc.)
     *
     * @param void
     * @return null
     */
    function import_project() {
    	if(!$this->request->isAsyncCall() || !$this->request->isSubmitted()) {
    		die('HTTP/1.1 400 Bad Request');
    	} // if
    	
    	set_time_limit(0);
    	
			$import_project_id = $this->request->post('import_project_id');
  		if(is_foreachable($import_projects = $this->importer->getImportFile('projects_list'))) {
  			$found = false;
  			foreach($import_projects as $import_project) {
  				if ($import_project['id'] == $import_project_id) {
  					$found = true;
  					break;
  				} // if
  			} // foreach
  			
  			if(!$found) {
  				die(lang('There is no selected project file to import'));
  			} // if
  		} // if
  		
  		// import selected project
			$this->import_selected_project($import_project);
			
			// import project users
			$project_people_file = Importer::getProjectFile('project_people', $import_project_id);
    	if($project_people_file_handle = fopen($project_people_file, 'r')) {
    		while(!feof($project_people_file_handle)) {
    			$line = fgets($project_people_file_handle);
    			$prepared_line = unserialize(base64_decode(trim($line)));
    			
    			if($prepared_line) {
    				$this->import_project_user($prepared_line);
    			} //
    		} // while
    		fclose($project_people_file_handle);
    	} else {
    		die(lang('Could not open project user file for reading: :name', array('name' => $project_people_file)));
    	} // if
    	
			// import project objects
    	$this->import_project_filename = Importer::getProjectFile('project_objects', $import_project_id);
    	if($this->file_handle = fopen($this->import_project_filename, 'r')) {
    		while(!feof($this->file_handle)) {
    			$line = fgets($this->file_handle);
    			$prepared_line = unserialize(base64_decode(trim($line)));
    			
    			if($prepared_line) {
	    			switch($prepared_line['type']) {
	    				case 'category':
	    					$this->import_project_category($prepared_line);
	    					break;
	    				case 'milestone':
	    					$this->import_milestone($prepared_line);
	    					break;
	    				case 'ticket':
	    					$this->import_ticket($prepared_line);
	    					break;
	    				case 'checklist':
	    					$this->import_ticket($prepared_line);
	    					break;
	    				case 'task':
	    					$this->import_task($prepared_line);
	    					break;
	    				case 'comment':
	    					$this->import_comment($prepared_line);
	    					break;
	    				case 'timerecord':
	    					$this->import_timerecord($prepared_line);
	    					break;
	    				case 'discussion':
	    					$this->import_discussion($prepared_line);
	    					break;
	    				case 'file':
	    					$this->import_file($prepared_line);
	    					break;
	    			} // switch
    			} // if
    		} // while
    		fclose($this->file_handle);
    	} else {
    		die(lang('Could not open selected project file for reading: :name', array('name' => $this->import_project_filename)));
    	} // if
    	
			// import project object assignees
    	$assignees_file = Importer::getProjectFile('assignees', $import_project_id);
    	if($assignees_file_handle = fopen($assignees_file, 'r')) {
    		while(!feof($assignees_file_handle)) {
    			$line = fgets($assignees_file_handle);
    			$prepared_line = unserialize(base64_decode(trim($line)));
    			
    			if($prepared_line) {
    				$this->import_assignee($prepared_line);
    			} // if
    		} // while
    		fclose($assignees_file_handle);
    	} else {
    		die(lang('Could not open assignees file for reading: ;name', array('name' => $assignees_file)));
    	} // if
  		
			// import project object attachments
    	$attachments_file = Importer::getProjectFile('attachments', $import_project_id);
    	if($attachments_file_handle = fopen($attachments_file, 'r')) {
    		while(!feof($attachments_file_handle)) {
    			$line = fgets($attachments_file_handle);
    			$prepared_line = unserialize(base64_decode(trim($line)));
    			
    			if($prepared_line) {
    				$this->import_attachment($prepared_line);
    			} // if
    		} // while
    		fclose($attachments_file_handle);
    	} else {
    		die(lang('Could not open attachments file for reading: :name', array('name' => $attachments_file)));
    	} // if
    	
    	// refresh project progress bar
    	$this->importer->refreshProjectProgressBar($this->imported_project_id);
    	
    	// set administrator who importing the data as a project participant
    	$this->importer->addUserToProject($this->logged_user, $this->imported_project_id);
			
  		$_SESSION['imported_projects'][] = $import_project_id;
  		
    	die(lang('true'));
    } // import_project
		
		/**
		 * Import selected project
		 *
		 * @param array $project
		 * @return null
		 */
		function import_selected_project($project) {
			$real_company_id = $this->importer->getRealCompanyId(array_var($project, 'company_id'));
			
			// import project
			try {
				$this->importer->prepare_and_execute('projects', array(
					'company_id' 					=> $real_company_id,
					'group_id' 						=> array_var($project, 'group_id', 1),
					'name'								=> array_var($project, 'name', null),
					'leader_id'						=> array_var($project, 'leader_id', $this->logged_user->getId()),
					'leader_name'					=> array_var($project, 'leader_id', $this->logged_user->getDisplayName()),
					'leader_email'				=> array_var($project, 'leader_id', $this->logged_user->getEmail()),
					'overview'						=> array_var($project, 'overview', null),
					'status'							=> array_var($project, 'status'),
					'default_visibility'	=> array_var($project, 'default_visibility', true),
					'created_on'					=> array_var($project, 'created_on', new DateTimeValue()),
					'created_by_id'				=> array_var($project, 'created_by_id', $this->logged_user->getId()),
					'created_by_name'			=> array_var($project, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($project, 'created_by_email', $this->logged_user->getEmail()),
					'updated_on'					=> array_var($project, 'updated_on', new DateTimeValue())
				));
			} catch(Exception $e) {
				die(lang('Failed to import project :name', array('name' => array_values($project, 'name'))));
			} // try
			
			$this->imported_project_id = db_last_insert_id();
			
			// update project search index
			try {
				$this->importer->updateSearchIndex($this->imported_project_id, 'Project', array(
					'name' 			=> array_var($project, 'name', null),
					'overview' 	=> array_var($project, 'overview', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index for project :name', array('name' => array_values($project, 'name'))));
			} // try
		} // import_selected_project
		
		/**
		 * Import project user
		 *
		 * @param array $project_user
		 * @return null
		 */
		function import_project_user($project_user) {
			$real_user_id = $this->importer->getRealUserId(array_var($project_user, 'id'));
			
			// import project user
			try {
				$this->importer->prepare_and_execute('project_users', array(
					'user_id' 			=> $real_user_id,
					'project_id' 		=> $this->imported_project_id,
					'permissions'		=> serialize(array_var($project_user, 'permissions'))
				));
			} catch(Exception $e) {
				die(lang('Failed to import project user with ID :id', array('id' => $real_user_id)));
			} // try
		} // import_project_user
		
		/**
		 * Import project Files / Discussions category
		 *
		 * @param array $category
		 * @return null
		 */
		function import_project_category($category) {
			$real_created_by_id = $this->importer->getRealUserId(array_var($category, 'created_by_id'));
			
			// import project category
			try {
				$this->importer->prepare_and_execute('project_objects', array(
					'type' 								=> ucfirst(array_var($category, 'type', 'category')),
					'module' 							=> array_var($category, 'module'),
					'project_id'					=> $this->imported_project_id,
					'name'								=> array_var($category, 'name', null),
					'state'								=> array_var($category, 'state', STATE_VISIBLE),
					'visibility'					=> array_var($category, 'visibility', VISIBILITY_NORMAL),
					'created_on'					=> array_var($category, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($category, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($category, 'created_by_email', $this->logged_user->getEmail()),
					'varchar_field_1'			=> array_var($category, 'module'),
					'version'							=> array_var($category, 'version', 1)
				));
			} catch(Exception $e) {
				die(lang('Failed to import project category :name', array('name' => array_var($category, 'name'))));
			} // try
			
			$imported_project_category_id = db_last_insert_id();
			
			// update project category search index
			try {
				$this->importer->updateSearchIndex($imported_project_category_id, 'ProjectObject', array(
					'name' => array_var($category, 'name', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index for project category :name', array('name' => array_var($category, 'name'))));
			} // try
			
			// update recent activities (to-do list items as tickets convert type only)
			if(array_var($category, 'module') == 'tickets') {
				try {
					$this->importer->prepare_and_execute('activity_logs', array(
						'type' 							=> 'ObjectCreatedActivityLog',
						'object_id' 				=> $imported_project_category_id,
						'project_id'				=> $this->imported_project_id,
						'created_on'				=> array_var($category, 'created_on', new DateTimeValue()),
						'created_by_id'			=> $real_created_by_id,
						'created_by_name'		=> array_var($category, 'created_by_name', $this->logged_user->getDisplayName()),
						'created_by_email'	=> array_var($category, 'created_by_email', $this->logged_user->getEmail())
					));
				} catch (Exception $e) {
					die(lang('Failed to update recent activities for Ticket category :name', array('name' => array_var($category, 'name', null))));
				} // try
			} // if
			 
			$this->project_objects_map[$category['id']] = array(
				'real_id' 	=> $imported_project_category_id,
				'type'			=> ucfirst(array_var($category, 'type', 'category'))
			);
			
			unset($imported_project_category_id);
		} // import_project_category
		
		/**
		 * Import Milestone
		 *
		 * @param array $milestone
		 * @return null
		 */
		function import_milestone($milestone) {
			$real_created_by_id = $this->importer->getRealUserId(array_var($milestone, 'created_by_id'));
			$is_completed = array_var($milestone, 'completed') === false ? 0 : 1;
			$real_completed_by_id = $this->importer->getRealUserId(array_var($milestone, 'completed_by_id'));
			
			// import milestone
			try {
				$this->importer->prepare_and_execute('project_objects', array(
					'type' 								=> ucfirst(array_var($milestone, 'type', 'milestone')),
					'module' 							=> array_var($milestone, 'type', 'milestone') . 's',
					'project_id'					=> $this->imported_project_id,
					'name'								=> array_var($milestone, 'name', null),
					'state'								=> array_var($milestone, 'state', STATE_VISIBLE),
					'visibility'					=> array_var($milestone, 'visibility', VISIBILITY_NORMAL),
					'priority'						=> array_var($milestone, 'priority', PRIORITY_NORMAL),
					'created_on'					=> array_var($milestone, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($milestone, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($milestone, 'created_by_email', $this->logged_user->getEmail()),
					'updated_on'					=> array_var($milestone, 'completed_on', null),
					'updated_by_id'				=> $real_completed_by_id,
					'due_on'							=> array_var($milestone, 'due_on'),
					'completed_on'				=> array_var($milestone, 'completed_on', new DateTimeValue()),
					'completed_by_id'			=> $real_completed_by_id,
					'completed_by_name'		=> array_var($milestone, 'completed_by_name', null),
					'completed_by_email'	=> array_var($milestone, 'completed_by_email', null),
					'date_field_1'				=> array_var($milestone, 'starts_on', array_var($milestone, 'due_on')),
					'version'							=> $is_completed + 1,
				));
			} catch (Exception $e) {
				die(lang('Failed to import milestone :name', array('name' => array_var($milestone, 'name', null))));				
			} // try
			
			$imported_milestone_id = db_last_insert_id();
			
			// update milestone search index
			try {
				$this->importer->updateSearchIndex($imported_milestone_id, 'ProjectObject', array(
					'name' => array_var($milestone, 'name', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index for milestone :name', array('name' => array_var($milestone, 'name', null))));
			} // try
			
			// update recent activities
			try {
				$this->importer->prepare_and_execute('activity_logs', array(
					'type' 								=> 'ObjectCreatedActivityLog',
					'object_id' 					=> $imported_milestone_id,
					'project_id'					=> $this->imported_project_id,
					'created_on'					=> array_var($milestone, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($milestone, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($milestone, 'created_by_email', $this->logged_user->getEmail())
				));
			} catch (Exception $e) {
				die(lang('Failed to update recent activities for milestone :name', array('name' => array_var($milestone, 'name', null))));
			} // try
			
			// update recent activities
			if($is_completed) {
				try {
					$this->importer->prepare_and_execute('activity_logs', array(
						'type' 								=> 'TaskCompletedActivityLog',
						'object_id' 					=> $imported_milestone_id,
						'project_id'					=> $this->imported_project_id,
						'created_on'					=> array_var($milestone, 'completed_on', new DateTimeValue()),
						'created_by_id'				=> $real_completed_by_id,
						'created_by_name'			=> array_var($milestone, 'completed_by_name', $this->logged_user->getDisplayName()),
						'created_by_email'		=> array_var($milestone, 'completed_by_email', $this->logged_user->getEmail())
					));
				} catch (Exception $e) {
					die(lang('Failed to update recent activities for milestone :name', array('name' => array_var($milestone, 'name', null))));				
				} // try
			} // if
			
			$this->project_objects_map[$milestone['id']] = array(
				'real_id'		=> $imported_milestone_id,
				'type'			=> ucfirst(array_var($milestone, 'type', 'milestone'))
			);
			
			unset($imported_milestone_id);
		} // import_milestone
		
		/**
		 * Import Ticket / Checklist
		 *
		 * @param array $ticket
		 * @return null
		 */
		function import_ticket($ticket) {
			$real_milestone_id = $this->importer->getRealParentId(array_var($ticket, 'milestone_id'), $this->project_objects_map);
			$real_parent_id = $this->importer->getRealParentId(array_var($ticket, 'parent_id'), $this->project_objects_map);
			$real_created_by_id = $this->importer->getRealUserId(array_var($ticket, 'created_by_id'));
			$is_completed = array_var($ticket, 'completed') === false ? 0 : 1;
			$real_completed_by_id = $this->importer->getRealUserId(array_var($ticket, 'completed_by_id'));
			
			// import ticket / checklist
			try {
				$this->importer->prepare_and_execute('project_objects', array(
					'type' 								=> ucfirst(array_var($ticket, 'type', 'ticket')),
					'module' 							=> array_var($ticket, 'type', 'ticket') . 's',
					'project_id'					=> $this->imported_project_id,
					'milestone_id'				=> $real_milestone_id,
					'parent_id'						=> $real_parent_id,
					'name'								=> array_var($ticket, 'name', null),
					'body'								=> array_var($ticket, 'body', null),
					'state'								=> array_var($ticket, 'state', STATE_VISIBLE),
					'visibility'					=> array_var($ticket, 'visibility', VISIBILITY_NORMAL),
					'priority'						=> array_var($ticket, 'priority', PRIORITY_NORMAL),
					'created_on'					=> array_var($ticket, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($ticket, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($ticket, 'created_by_email', $this->logged_user->getEmail()),
					'updated_on'					=> array_var($ticket, 'completed_on', null),
					'due_on'							=> array_var($ticket, 'due_on', null),
					'updated_by_id'				=> $real_completed_by_id,
					'completed_on'				=> array_var($ticket, 'completed_on', new DateTimeValue()),
					'completed_by_id'			=> $real_completed_by_id,
					'completed_by_name'		=> array_var($ticket, 'completed_by_name', null),
					'completed_by_email'	=> array_var($ticket, 'completed_by_email', null),
					'integer_field_1'			=> array_var($ticket, 'number', $this->counter),
					'position'						=> array_var($ticket, 'position'),
					'version'							=> $is_completed + 1,
				));
			} catch (Exception $e) {
				die(lang('Failed to import ticket :name', array('name' => array_var($ticket, 'name', null))));
			} // try
			
			$imported_ticket_id = db_last_insert_id();
			
			// update ticket / checklist search index
			try {
				$this->importer->updateSearchIndex($imported_ticket_id, 'ProjectObject', array(
					'name' 	=> array_var($ticket, 'name', null),
					'body'	=> array_var($ticket, 'body', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index for ticket :name', array('name' => array_var($ticket, 'name', null))));
			} // try
			
			// update recent activities
			try {
				$this->importer->prepare_and_execute('activity_logs', array(
					'type' 								=> 'ObjectCreatedActivityLog',
					'object_id' 					=> $imported_ticket_id,
					'project_id'					=> $this->imported_project_id,
					'created_on'					=> array_var($ticket, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($ticket, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($ticket, 'created_by_email', $this->logged_user->getEmail())
				));
			} catch (Exception $e) {
				die(lang('Failed to update recent activities for ticket :name', array('name' => array_var($ticket, 'name', null))));				
			} // try
			
			// update recent activities
			if($is_completed) {
				try {
					$this->importer->prepare_and_execute('activity_logs', array(
						'type' 								=> 'TaskCompletedActivityLog',
						'object_id' 					=> $imported_ticket_id,
						'project_id'					=> $this->imported_project_id,
						'created_on'					=> array_var($ticket, 'completed_on', new DateTimeValue()),
						'created_by_id'				=> $real_completed_by_id,
						'created_by_name'			=> array_var($ticket, 'completed_by_name', $this->logged_user->getDisplayName()),
						'created_by_email'		=> array_var($ticket, 'completed_by_email', $this->logged_user->getEmail())
					));
				} catch (Exception $e) {
					die(lang('Failed to update recent activities for ticket :name', array('name' => array_var($ticket, 'name', null))));				
				} // try
			} // if
			
			$this->project_objects_map[$ticket['id']] = array(
				'real_id'	=> $imported_ticket_id,
				'type'		=> ucfirst(array_var($ticket, 'type', 'ticket'))
			);
			$this->counter++;
			
			unset($imported_ticket_id);
		} // import_ticket
		
		/**
		 * Import Task
		 *
		 * @param array $task
		 * @return null
		 */
		function import_task($task) {
			$real_milestone_id = $this->importer->getRealParentId(array_var($task, 'milestone_id'), $this->project_objects_map);
			$real_parent_id = $this->importer->getRealParentId(array_var($task, 'parent_id'), $this->project_objects_map);
			$real_created_by_id = $this->importer->getRealUserId(array_var($task, 'created_by_id'));
			$is_completed = array_var($task, 'completed') === false ? 0 : 1;
			$real_completed_by_id = $this->importer->getRealUserId(array_var($task, 'completed_by_id'));
			
			// import task
			try {
				$this->importer->prepare_and_execute('project_objects', array(
					'type' 								=> ucfirst(array_var($task, 'type', 'task')),
					'module' 							=> 'resources',
					'project_id'					=> $this->imported_project_id,
					'milestone_id'				=> $real_milestone_id,
					'parent_id'						=> $real_parent_id,
					'parent_type'					=> array_var($task, 'parent_type'),
					'body'								=> array_var($task, 'body', null),
					'state'								=> array_var($task, 'state', STATE_VISIBLE),
					'visibility'					=> array_var($task, 'visibility', VISIBILITY_NORMAL),
					'priority'						=> array_var($task, 'priority', PRIORITY_NORMAL),
					'created_on'					=> array_var($task, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($task, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($task, 'created_by_email', $this->logged_user->getEmail()),
					'due_on'							=> array_var($task, 'due_on', null),
					'updated_on'					=> array_var($task, 'completed_on', null),
					'updated_by_id'				=> $real_completed_by_id,
					'completed_on'				=> array_var($task, 'completed_on', new DateTimeValue()),
					'completed_by_id'			=> $real_completed_by_id,
					'completed_by_name'		=> array_var($task, 'completed_by_name', null),
					'completed_by_email'	=> array_var($task, 'completed_by_email', null),
					'position'						=> array_var($task, 'position'),
					'version'							=> $is_completed + 1,
				));
			} catch (Exception $e) {
				die(lang('Failed to import task :body', array('body' => array_var($task, 'body', null))));
			} // try
			
			$imported_task_id = db_last_insert_id();
			
			// update task search index
			try {
				$this->importer->updateSearchIndex($imported_task_id, 'ProjectObject', array(
					'body' => array_var($task, 'body', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index for task :body', array('body' => array_var($task, 'body', null))));
			} // try
			
			// update recent activities
			try {
				$this->importer->prepare_and_execute('activity_logs', array(
					'type' 								=> 'NewTaskActivityLog',
					'object_id' 					=> $imported_task_id,
					'project_id'					=> $this->imported_project_id,
					'created_on'					=> array_var($task, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($task, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($task, 'created_by_email', $this->logged_user->getEmail())
				));
			} catch (Exception $e) {
				die(lang('Failed to update recent activities for task :body', array('body' => array_var($task, 'body', null))));
			} // try
			
			// update recent activities
			if($is_completed) {
				try {
					$this->importer->prepare_and_execute('activity_logs', array(
						'type' 								=> 'TaskCompletedActivityLog',
						'object_id' 					=> $imported_task_id,
						'project_id'					=> $this->imported_project_id,
						'created_on'					=> array_var($task, 'completed_on', new DateTimeValue()),
						'created_by_id'				=> $real_completed_by_id,
						'created_by_name'			=> array_var($task, 'completed_by_name', $this->logged_user->getDisplayName()),
						'created_by_email'		=> array_var($task, 'completed_by_email', $this->logged_user->getEmail())
					));
				} catch (Exception $e) {
					die(lang('Failed to update recent activities for completed task :body', array('body' => array_var($task, 'body', null))));
				} // try
			} // if
			
			$this->project_objects_map[$task['id']] = array(
				'real_id' => $imported_task_id,
				'type'		=> ucfirst(array_var($ticket, 'type', 'ticket'))
			);
			
			unset($imported_task_id);
		} // import_task
		
		/**
		 * Import Comment
		 *
		 * @param array $comment
		 * @return null
		 */
		function import_comment($comment) {
			$real_parent_id = $this->importer->getRealParentId(array_var($comment, 'parent_id'), $this->project_objects_map);
			$real_created_by_id = $this->importer->getRealUserId(array_var($comment, 'created_by_id'));
			
			// import comment
			try {
				$this->importer->prepare_and_execute('project_objects', array(
					'type' 								=> ucfirst(array_var($comment, 'type', 'comment')),
					'module' 							=> 'resources',
					'project_id'					=> $this->imported_project_id,
					'parent_id'						=> $real_parent_id,
					'parent_type'					=> array_var($comment, 'parent_type'),
					'body'								=> array_var($comment, 'body', null),
					'state'								=> array_var($comment, 'state', STATE_VISIBLE),
					'visibility'					=> array_var($comment, 'visibility', VISIBILITY_NORMAL),
					'created_on'					=> array_var($comment, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($comment, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($comment, 'created_by_email', $this->logged_user->getEmail()),
					'version'							=> array_var($comment, 'vesion', 1)
				));
			} catch (Exception $e) {
				die(lang('Failed to import ' . array_var($comment, 'parent_type') . ' comment :body', array('body' => array_var($comment, 'body', null))));
			} // try
			
			$imported_comment_id = db_last_insert_id();
			
			// update comment search index
			try {
				$this->importer->updateSearchIndex($imported_comment_id, 'ProjectObject', array(
					'body' => array_var($comment, 'body', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index for ' . array_var($comment, 'parent_type') . ' comment :body', array('body' => array_var($comment, 'body', null))));
			} // try
			
			// update recent activities
			try {
				$this->importer->prepare_and_execute('activity_logs', array(
					'type' 								=> 'NewCommentActivityLog',
					'object_id' 					=> $imported_comment_id,
					'project_id'					=> $this->imported_project_id,
					'created_on'					=> array_var($comment, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($comment, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($comment, 'created_by_email', $this->logged_user->getEmail())
				));
			} catch (Exception $e) {
				die(lang('Failed to update recent activities for ' . array_var($comment, 'parent_type') . ' comment :body', array('body' => array_var($comment, 'body', null))));
			} // try
			
			$this->project_objects_map[$comment['id']] = array(
				'real_id' => $imported_comment_id,
				'type'		=> ucfirst(array_var($comment, 'type', 'comment'))
			);
			
			unset($imported_comment_id);
		} // import_comment
		
		/**
		 * Import Time Record
		 *
		 * @param array $timerecord
		 * @return null
		 */
		function import_timerecord($timerecord) {
			$real_parent_id = $this->importer->getRealParentId(array_var($timerecord, 'parent_id'), $this->project_objects_map);
			$real_tracked_user_id = $this->importer->getRealUserId(array_var($timerecord, 'integer_field_1'));
			
			// import time record
			try {
				$this->importer->prepare_and_execute('project_objects', array(
					'type' 								=> 'TimeRecord',
					'module' 							=> 'timetracking',
					'project_id'					=> $this->imported_project_id,
					'parent_id'						=> $real_parent_id,
					'parent_type'					=> array_var($timerecord, 'parent_type', null),
					'body'								=> array_var($timerecord, 'body', null),
					'state'								=> array_var($timerecord, 'state', STATE_VISIBLE),
					'visibility'					=> array_var($timerecord, 'visibility', VISIBILITY_NORMAL),
					'created_on'					=> array_var($timerecord, 'created_on', new DateTimeValue()),
					'created_by_id'				=> array_var($timerecord, 'created_by_id', $this->logged_user->getId()),
					'created_by_name'			=> array_var($timerecord, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($timerecord, 'created_by_email', $this->logged_user->getEmail()),
					'varchar_field_1'			=> array_var($timerecord, 'tracked_user_name'),
					'varchar_field_2'			=> array_var($timerecord, 'tracked_user_email'),
					'integer_field_1'			=> $real_tracked_user_id,
					'integer_field_2'			=> array_var($timerecord, 'integer_field_2', 1),
					'float_field_1'				=> array_var($timerecord, 'float_field_1'),
					'date_field_1'				=> array_var($timerecord, 'date_field_1'),
					'version'							=> array_var($timerecord, 'vesion', 1)
				));
			} catch (Exception $e) {
				die(lang('Failed to imoport time record'));
			} // try
			
			$imported_time_id = db_last_insert_id();
			
			// update timerecord search index
			try {
				$this->importer->updateSearchIndex($imported_time_id, 'ProjectObject', array(
					'body' => array_var($timerecord, 'body', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index time record'));
			} // try
			
			// mark parent object that has time
			try {
				$this->importer->updateTimeRecordParentObject($real_parent_id);
			} catch(Exception $e) {
				die(lang('Failed to update proper time record parent object'));
			} // try
			
			unset($imported_time_id);
		} // import_timerecord
		
		/**
		 * Import Discussion
		 *
		 * @param array $discussion
		 * @return null
		 */
		function import_discussion($discussion) {
			$real_milestone_id = $this->importer->getRealParentId(array_var($discussion, 'milestone_id'), $this->project_objects_map);
			$real_parent_id = $this->importer->getRealParentId(array_var($discussion, 'parent_id'), $this->project_objects_map);
			$real_created_by_id = $this->importer->getRealUserId(array_var($discussion, 'created_by_id'));
			
			// import discussion
			try {
				$this->importer->prepare_and_execute('project_objects', array(
					'type' 								=> ucfirst(array_var($discussion, 'type', 'discussion')),
					'module' 							=> array_var($discussion, 'type', 'discussion') . 's',
					'project_id'					=> $this->imported_project_id,
					'milestone_id'				=> $real_milestone_id,
					'parent_id'						=> $real_parent_id,
					'name'								=> array_var($discussion, 'name', null),
					'body'								=> array_var($discussion, 'body', null),
					'state'								=> array_var($timerecord, 'state', STATE_VISIBLE),
					'visibility'					=> array_var($discussion, 'visibility', VISIBILITY_NORMAL),
					'created_on'					=> array_var($discussion, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($discussion, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($discussion, 'created_by_email', $this->logged_user->getEmail()),
					'comments_count'			=> array_var($discussion, 'comments_count'),
					'datetime_field_1'		=> array_var($discussion, 'created_on', new DateTimeValue()),
					'version'							=> array_var($discussion, 'vesion', 1)
				));
			} catch (Exception $e) {
				die(lang('Failed to import discussion :name', array('name' => array_var($discussion, 'name'))));
			} // try
			
			$imported_discussion_id = db_last_insert_id();
			
			// update discussion search index
			try {
				$this->importer->updateSearchIndex($imported_discussion_id, 'ProjectObject', array(
					'name' 	=> array_var($discussion, 'name', null),
					'body'	=> array_var($discussion, 'body', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index for discussion :name', array('name' => array_var($discussion, 'name'))));
			} // try
			
			// update recent activities
			try {
				$this->importer->prepare_and_execute('activity_logs', array(
					'type' 								=> 'ObjectCreatedActivityLog',
					'object_id' 					=> $imported_discussion_id,
					'project_id'					=> $this->imported_project_id,
					'created_on'					=> array_var($discussion, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($discussion, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($discussion, 'created_by_email', $this->logged_user->getEmail())
				));
			} catch (Exception $e) {
				die(lang('Failed to update recent activities for discussion :name', array('name' => array_var($discussion, 'name'))));
			} // try

			$this->project_objects_map[$discussion['id']] = array(
				'real_id' => $imported_discussion_id,
				'type'		=> ucfirst(array_var($discussion, 'type', 'discussion'))
			);
			
			unset($imported_discussion_id);
		} // import_discussion
		
		/**
     * Import File
     *
     * @param array $file
     * @return null
     */
    function import_file($file) {
    	$real_parent_id = $this->importer->getRealParentId(array_var($file, 'parent_id'), $this->project_objects_map);
			$real_created_by_id = $this->importer->getRealUserId(array_var($file, 'created_by_id'));
			$real_updated_by_id = $this->importer->getRealUserId(array_var($file, 'updated_by_id'));
			
			$name = rawurldecode(array_var($file, 'name', null));
			
			// import file
			try {
				$this->importer->prepare_and_execute('project_objects', array(
					'type' 								=> ucfirst(array_var($file, 'type', 'file')),
					'module' 							=> array_var($file, 'type', 'file') . 's',
					'project_id'					=> $this->imported_project_id,
					'parent_id'						=> $real_parent_id,
					'parent_type'					=> array_var($file, 'parent_type', null),
					'name'								=> $name,
					'body'								=> array_var($file, 'body', null),
					'state'								=> array_var($file, 'state', STATE_VISIBLE),
					'visibility'					=> array_var($file, 'visibility', VISIBILITY_NORMAL),
					'created_on'					=> array_var($file, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($file, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($file, 'created_by_email', $this->logged_user->getEmail()),
					'updated_on'					=> array_var($file, 'updated_on', new DateTimeValue()),
					'updated_by_id'				=> $real_updated_by_id,
					'updated_by_name'			=> array_var($file, 'updated_by_name', $this->logged_user->getDisplayName()),
					'updated_by_email'		=> array_var($file, 'updated_by_email', $this->logged_user->getEmail()),
					'integer_field_1'			=> array_var($file, 'integer_field_1', 1),
					'version'							=> array_var($file, 'version', 1)
				));
			} catch(Exception $e) {
				die(lang('Failed to import file :name', array('name' => array_var($file, 'name', null))));
			} // try
			
			$imported_file_id = db_last_insert_id();
			
			// update file search index
			try {
				$this->importer->updateSearchIndex($imported_file_id, 'ProjectObject', array(
					'name' 	=> $name,
					'body'	=> array_var($file, 'body', null)
				));
			} catch(Exception $e) {
				die(lang('Failed to update search index for file :name', array('name' => array_var($file, 'name', null))));
			} // try
			
			// update recent activities
			try {
				$this->importer->prepare_and_execute('activity_logs', array(
					'type' 								=> 'NewFileActivityLog',
					'object_id' 					=> $imported_file_id,
					'project_id'					=> $this->imported_project_id,
					'created_on'					=> array_var($file, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($file, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($file, 'created_by_email', $this->logged_user->getEmail())
				));
			} catch (Exception $e) {
				die(lang('Failed to update recent activities for file :name', array('name' => array_var($file, 'name', null))));
			} // try
			
			$this->project_objects_map[$file['id']] = array(
				'real_id' 	=> $imported_file_id,
				'type'			=> ucfirst(array_var($file, 'type', 'file'))
			);
			
			unset($imported_file_id);
    } // import_file
		
		/**
     * Import assignee
     *
     * @param array $assignee
     * @return null
     */
    function import_assignee($assignee) {
			$real_user_id = $this->importer->getRealUserId(array_var($assignee, 'user_id'));
			$real_object_id = $this->importer->getRealParentId(array_var($assignee, 'object_id'), $this->project_objects_map);
			
			// import assignee
			try {
				$this->importer->prepare_and_execute('assignments', array(
					'user_id' 	=> $real_user_id,
					'object_id'	=> $real_object_id,
					'is_owner'	=> array_var($assignee, 'is_owner')
				));
			} catch (Exception $e) {
				die(lang('Failed to import assignees'));
			} // try
    } // import_assignee
    
    /**
     * Import attachment
     *
     * @param array $attachment
     * @return null
     */
    function import_attachment($attachment) {
    	$real_parent_id = $this->importer->getRealParentId(array_var($attachment, 'parent_id'), $this->project_objects_map);
			$real_created_by_id = $this->importer->getRealUserId(array_var($attachment, 'created_by_id'));
			
			$import_name = $attachment['id'].'_'.$attachment['name'];
			$import_path = IMPORT_PATH . '/uploads/' . $import_name;
			
			$name = rawurldecode(array_var($attachment, 'name', null));
			
			if($this->fileinfo_loaded) {
				$finfo = new finfo(FILEINFO_MIME_TYPE);
				$mime_type = $finfo->file($import_path);
			} else {
				$mime_type = $this->importer->get_mime_type($import_path);
			} // if
			
			$size = filesize($import_path);
			
			$destination_path = get_available_uploads_filename();
			$location = basename($destination_path);
			
			// import attachment
			try {
				$this->importer->prepare_and_execute('attachments', array(
					'parent_id'						=> $real_parent_id,
					'parent_type'					=> ucfirst(array_var($attachment, 'parent_type', 'file')),
					'name'								=> $name,
					'mime_type'						=> $mime_type,
					'size'								=> $size,
					'location'						=> $location,
					'attachment_type'			=> array_var($attachment, 'attachment_type', 'file_revision'),
					'created_on'					=> array_var($attachment, 'created_on', new DateTimeValue()),
					'created_by_id'				=> $real_created_by_id,
					'created_by_name'			=> array_var($attachment, 'created_by_name', $this->logged_user->getDisplayName()),
					'created_by_email'		=> array_var($attachment, 'created_by_email', $this->logged_user->getEmail())
				));
			
				if (!copy($import_path, $destination_path)) {
					throw new Exception(lang('Could not move import file to the uploads folder (:import_file)', array('import_file' => $import_path)));
				} // if
			} catch(Exception $e) {
				die(lang('Failed to import attachment :name', array('name' => array_var($attachment, 'name', null))));
			} // try
			
			$this->project_objects_map[$attachment['id']] = array(
				'real_id' 	=> db_last_insert_id(),
				'type'			=> ucfirst(array_var($attachment, 'type', 'attachment'))
			);
    } // import_attachment
		
  }
  
?>