<?php

  /**
   * Cron controller
   * 
   * This controller is used to trigger daily and hourly tasks through web 
   * interface
   *
   * @package activeCollab.modules.system
   * @subpackage controllers
   */
  class CronController extends PageController {
    
    /**
     * Default flag for generating usage stats
     *
     * @var boolean
     */
    var $generate_stats = false;
    
    /**
     * Format the response according to the requested data type
     *
     * @var string
     */
    var $format_type = null;
    
    /**
     * Benchmark for the cron job
     *
     * @var BenchmarkTimer
     */
    var $benchmark = null;
    
    /**
     * Construct cron controller
     *
     * @param Request $request
     * @return null
     */
    function __construct($request) {
      parent::__construct($request);
      
      $this->format_type = $this->request->get('format_type') == "json" ? FORMAT_JSON : FORMAT_XML;
      $this->benchmark = new BenchmarkTimer();
      
      $code = $this->request->get('code');
      if(defined('PROTECT_SCHEDULED_TASKS') && PROTECT_SCHEDULED_TASKS) {
        if(empty($code) || strtoupper($code) != strtoupper(substr(LICENSE_KEY, 0, 5))) {
          die('dafak');
          $this->httpError(HTTP_ERR_FORBIDDEN);
        } // if
      } // if
      
      // generate stats only if we have the code supplied
      $this->generate_stats = strtoupper($code) == strtoupper(substr(LICENSE_KEY, 0, 5)) && !is_null($this->request->get('stats'));
    } // __construct
    
    /**
     * Trigger frequently event
     *
     * @param void
     * @return null
     */
    function frequently() {
      $this->benchmark->start();
      event_trigger('on_frequently');
      $this->benchmark->stop();
      
      ConfigOptions::setValue('last_frequently_activity', time());
      
      $event = array(
        "message"   => "Frequently event executed",
        "duration"  => $this->benchmark->timeElapsed()
      );
      $this->serveData($event, null, $this->format_type);
    } // frequently
    
    /**
     * Trigger hourly tasks
     *
     * @param void
     * @return null
     */
    function hourly() {
      $this->benchmark->start();
    	event_trigger('on_hourly');
    	$this->benchmark->stop();
    	
    	ConfigOptions::setValue('last_hourly_activity', time());
    	
    	$event = array(
        "message"   => "Hourly event executed",
        "duration"  => $this->benchmark->timeElapsed()
      );
    	$this->serveData($event, null, $this->format_type);
    } // hourly
    
    /**
     * Trigger daily tasks
     *
     * @param void
     * @return null
     */
    function daily() {
      $this->benchmark->start();
    	event_trigger('on_daily');
    	$this->benchmark->stop();
    	
    	ConfigOptions::setValue('last_daily_activity', time());
    	
    	$event = array(
        "message"   => "Daily event executed",
        "duration"  => $this->benchmark->timeElapsed()
      );
    	
    	if ($this->generate_stats) {
    	  $today = new DateTimeValue();
    	  $application = &application();
      	$stats = array(
      	  'projects'         => (integer) array_var(db_execute_one("SELECT COUNT(*) AS total FROM ".TABLE_PREFIX."projects"), 'total', 0),
      	  'users'            => (integer) array_var(db_execute_one("SELECT COUNT(*) AS total FROM ".TABLE_PREFIX."users"), 'total', 0),
      	  'companies'        => (integer) array_var(db_execute_one("SELECT COUNT(*) AS total FROM ".TABLE_PREFIX."companies"), 'total', 0),
      	  'project_objects'  => (integer) array_var(db_execute_one("SELECT COUNT(*) AS total FROM ".TABLE_PREFIX."project_objects"), 'total', 0),
      	  'activity_logs'    => (integer) array_var(db_execute_one("SELECT COUNT(*) AS total FROM ".TABLE_PREFIX."activity_logs"), 'total', 0),
      	  'active'           => array_var(db_execute_one("SELECT COUNT(*) AS total FROM ".TABLE_PREFIX."users WHERE last_activity_on >= '{$today->beginningOfDay()}' AND last_activity_on <= '{$today->endOfDay()}'"), 'total', 0) > 0,
          'ac_version'       => $application->version,
          'php_version'      => phpversion(),
          'mysql_version'    => array_var(db_execute_one("SELECT VERSION() AS db_version"), 'db_version'),
          'innodb_support'   => defined('DB_CAN_TRANSACT') && DB_CAN_TRANSACT
      	);
    	} // if
    	
    	// if we are sending stats also, then merge everything into one array
    	$data = $this->generate_stats ? array_merge($event, array('stats'=>$stats)) : $event;
    	
    	$this->serveData($data, null, $this->format_type);
    } // daily
    
  }

?>