<?php

  /**
   * Thumbnails manager
   * 
   * This class is used to create, store and link to thumbnails generated by 
   * activeCollab
   *
   * @package activeCollab.modules.system
   * @subpackage models
   */
  class Thumbnails {
  
    /**
     * Create a thumbnail from a given file
     * 
     * This function will create a new thumbnail based on input file and return 
     * path of generated thumbnail on success (FALSE othervise). If thumbnails 
     * already exists it will return its path
     *
     * @param string $path
     * @param string $prefix
     * @param integer $max_width
     * @param integer $max_height
     * @return string
     */
    function create($path, $prefix, $max_width, $max_height) {
      $destination = THUMBNAILS_PATH . '/' . $prefix . '-' . basename($path);
      if(is_file($destination)) {
        return $destination;
      } else {
        if(scale_image($path, $destination, $max_width, $max_height, IMAGETYPE_JPEG)) {
          return $destination;
        } // if
      } // if
      return false;
    } // create
    
    /**
     * Return URL for a given file
     * 
     * If thumbnail does not exist false will be returned
     *
     * @param string $filename
     * @return string
     */
    function getUrl($filename) {
      $path = THUMBNAILS_PATH . '/' . basename($filename);
      if(is_file($path)) {
        return ROOT_URL . '/thumb.php?name=' . $filename . '&ver=' . filesize($path);
      } // if
      return false;
    } // getUrl
  
  } // Thumbnails

?>