App.importer = {
  controllers : {},
  models      : {}
};

/**
 * Importer admin behaviour
 */
App.importer.controllers.importer_admin = {
	
	/**
   * Index page bahaviour
   */
	index : function() {
		$(document).ready(function() {
			// if companies & users aren't imported yet, don't allow import of
			// default categories and projects
			if(!App.data.is_companies_imported) {
				$('.cat_and_proj').hide();
			} // if
			
			App.importer.importCategories.init('categories');
			App.importer.importCompanies.init('companies');
			App.importer.importAllProjects.init();
			App.importer.importProject.init('project');
		});
	}
	
};

/**
 * Import all projects
 */
App.importer.importAllProjects = function() {
	
	/**
   * All project IDs
   *
   * @var array
   */
	var project_ids = [];
	
	/**
   * project_ids variable's array element number
   *
   * @var integer
   */
  var next_project = 0;
  
  /**
   * Import all button image placeholder
   *
   * @var jQuery
   */
  var import_all_indicator_image;
	
	// Public interface
	return {
		
		/**
     * Initialize projects importing
     */
		init : function() {
			$('.import_all').find('a').click(function() {
				var link = $(this);
				var wrapper = link.parent();
				
				link.hide();
				wrapper.prepend('<img src="' + App.data.indicator_url  + '" alt="" />');
				import_all_indicator_image = $('img:eq(0)', wrapper);
				
				$('.project').each(function() {
					var project_row = $(this);
					var project_link = project_row.find('a');
					
					// prevents projects re-importing
					if(project_link.is('a')) {
						project_ids.push(project_link.attr('id'));
					} // if
					project_link.hide().parent().css('padding', '18px 0');
				});
				
				if(project_ids.length > 0) {
					App.importer.importAllProjects.import_projects();
				} // if
			});
		}, // init
		
		/**
     * Import projects
     */
    import_projects : function() {
    	if(next_project == project_ids.length) {
    		import_all_indicator_image.attr('src', App.data.ok_indicator_url);
	    	$('.all_done').css('display', 'block');
    	} else {
    		var next_project_id = project_ids[next_project];
	      var import_project_link = $('.project').find('a#' + next_project_id);
				var import_project_link_wrapper = import_project_link.parent();
	
				import_project_link_wrapper.css('padding', '5px 0').prepend('<img src="' + App.data.indicator_url  + '" alt="" />');
				var import_project_indicator_image = $('img:eq(0)', import_project_link_wrapper);
				
				$.ajax({
					url : App.extendUrl(App.data.import_project_url, {'async' : 1}),
					type : 'POST',
					data : {'submitted' : 'submitted', 'import_project_id' : next_project_id},
					success : function(response) {
						if(response == 'true') {
							import_project_indicator_image.attr('src', App.data.ok_indicator_url);
							next_project = next_project + 1;
	            App.importer.importAllProjects.import_projects();
						} else {
							import_project_indicator_image.attr('src', App.data.error_indicator_url);
							import_project_link_wrapper.parent().after('<tr style="color: red"><td>Error: ' + response + '</td><td class="options"></td></tr>');
							import_all_indicator_image.attr('src', App.data.error_indicator_url);
							
							return false;
						} // if
					}
				});
    	} // if
    } // import_projects
		
	};
	
}(); // importAllProjects

/**
 * Import companies and users module
 */
App.importer.importCompanies = function() {
	
	/**
   * Companies row
   *
   * @var jQuery
   */
	var companies_row;
	
	// Public interface
	return {
		
		init : function(companies_row_class) {
			companies_row = $('.' + companies_row_class);
			
			companies_row.find('a').click(function() {
				var link = $(this);
				var wrapper = link.parent();
				var row = wrapper.parent().parent();
				
				link.hide();
				wrapper.prepend('<img src="' + App.data.indicator_url  + '" alt="" />');
				var indicator_image = $('img:eq(0)', wrapper);
				
				// firstly create the database backup
				$.ajax({
					url 		: App.extendUrl(App.data.create_backup_url, {'async' : 1}),
					type 		: 'POST',
					data 		: {'submitted' : 'submitted'},
					success : function(response) {
						if(response == 'ok') {
							// grab selected role values
							var role = [$('#isAdmin').val(), $('#canSeePrivate').val(), $('#cannotSeePrivate').val()];
							
							// import companies & users
							$.ajax({
								url 		: App.extendUrl(App.data.import_companies_url, {'async' : 1}),
								type 		: 'POST',
								data 		: "submitted=submitted&role[admin_role_id]=" + role[0] + "&role[can_see_private_role_id]=" + role[1] + "&role[cannot_see_private_role_id]=" + role[2],
								success : function(response) {
									if(response == 'ok') {
										indicator_image.attr('src', App.data.ok_indicator_url);
										$('.cat_and_proj').show('fast');
									} else {
										indicator_image.attr('src', App.data.error_indicator_url);
										row.after('<tr style="color: red"><td>Error: ' + response + '</td><td class="options"></td></tr>');
									} // if
								}
							});
						} else {
							indicator_image.attr('src', App.data.error_indicator_url);
							row.after('<tr style="color: red"><td>Error: ' + response + '</td><td class="options"></td></tr>');
							
							return false;
						} // if
					}
				});
			});
		} // init
		
	};
	
}();

/**
 * Import master categories module
 */
App.importer.importCategories = function() {
	
	/**
   * Categories row
   *
   * @var jQuery
   */
	var categories_row;
	
	// Public interface
	return {
		
		init : function(categories_row_class) {
			categories_row = $('.' + categories_row_class);
			
			categories_row.find('a').click(function() {
				var link = $(this);
				var wrapper = link.parent();
				var row = wrapper.parent().parent();
				
				link.hide();
				wrapper.prepend('<img src="' + App.data.indicator_url  + '" alt="" />');
				var indicator_image = $('img:eq(0)', wrapper);
				
				$.ajax({
					url : App.extendUrl(App.data.import_categories_url, {'async' : 1}),
					type : 'POST',
					data : {'submitted' : 'submitted'},
					success : function(response) {
						if(response == 'ok') {
							indicator_image.attr('src', App.data.ok_indicator_url);
						} else {
							indicator_image.attr('src', App.data.error_indicator_url);
							row.after('<tr style="color: red"><td>Error: ' + response + '</td><td class="options"></td></tr>');
						} // if
					}
				});
			});
		} // init
		
	};
	
}();

/**
 * Import project and all its related object (milestones, tickets, etc.)
 */
App.importer.importProject = function() {
	
	/**
   * Single project row
   *
   * @var jQuery
   */
	var project_row;
	
	// Public interface
	return {
		
		init : function(project_row_class) {
			project_row = $('.' + project_row_class);
			
			project_row.each(function() {
				var row = $(this);
				
				row.find('a').click(function() {
					var link = $(this);
					var wrapper = link.parent();
					
					link.hide();
					wrapper.prepend('<img src="' + App.data.indicator_url  + '" alt="" />');
					var indicator_image = $('img:eq(0)', wrapper);
					
					$.ajax({
						url : App.extendUrl(App.data.import_project_url, {'async' : 1}),
						type : 'POST',
						data : {'submitted' : 'submitted', 'import_project_id' : link.attr('id')},
						success : function(response) {
							if(response == 'true') {
								indicator_image.attr('src', App.data.ok_indicator_url);
							} else {
								indicator_image.attr('src', App.data.error_indicator_url);
								row.after('<tr style="color: red"><td>Error: ' + response + '</td><td class="options"></td></tr>');
							} // if
						}
					});
				});
				
			});
		} // init
		
	};
	
}();